#! /usr/bin/python

'''
Notice:
THIS DOCUMENT IS PROVIDED "AS IS" WITH NO REPRESENTATION OR WARRANTY, 
EXPRESS OR IMPLIED, CONCERNING THE COMPLETENESS, ACCURACY, OR 
APPLICABILITY OF ANY INFORMATION CONTAINED IN THIS DOCUMENT. 
INTERTRUST TECHNOLOGIES CORPORATION,  
KONINKLIJKE PHILIPS ELECTRONICS N.V., 
MATSUSHITA ELECTRIC INDUSTRIAL CO., LTD., 
SAMSUNG ELECTRONICS CO., LTD., AND 
SONY CORPORATION (COLLECTIVELY, THE "PARTIES") DISCLAIM ALLLIABILITY 
OF ANY KIND WHATSOEVER, EXPRESS OR IMPLIED, ARISING OR RESULTING FROM
THE RELIANCE OR USE BY ANY PARTY OF THIS DOCUMENT OR ANY INFORMATION
CONTAINED HEREIN. THE PARTIES COLLECTIVELY AND INDIVIDUALLY MAKE NO 
REPRESENTATIONS CONCERNING THE APPLICABILITY OF ANY PATENT, COPYRIGHT 
OR OTHER PROPRIETARY RIGHT OF THIS DOCUMENT OR ITS USE, AND THE RECEIPT 
OR ANY USEOF THIS DOCUMENT OR ITS CONTENTS DOES NOT IN ANY WAY CREATE 
BY IMPLICATION, ESTOPPEL OR OTHERWISE, ANY LICENSE OR RIGHT TO OR UNDER 
ANY PATENT, COPYRIGHT, TRADEMARK OR TRADE SECRET RIGHTS WHICH ARE OR 
MAY BE ASSOCIATED WITH THE IDEAS, TECHNIQUES, CONCEPTS OR EXPRESSIONS 
CONTAINED HEREIN.
Use of this document is subject to the agreement executed between you and 
the Parties.

Copyright  2006 - 2007 by Intertrust Technologies Corporation, 
Koninklijke Philips Electronics N.V., 
Matsushita Electric Industrial Co., Ltd., 
Samsung Electronics Co., Ltd., and Sony Corporation . 
Third-party brands and names are the property of their respective owners.

Intellectual Property
A Commercial Implementation of this specification requires a license from 
the Marlin Trust Management Organization.

Contact Information
Feedback on this specification should be addressed to: 
editor@marlin-community.com
Contact information for the Marlin Trust Management Organization can be 
found at: http://www.marlin-trust.com
'''

'''
Conformance Test Suite and Reference Implementation for Marlin URI Templates
----------------------------------------------------------------------------

This python script contains a reference implementation of the Marlin URI
Templates specification. It also contains a series of test vectors that can
be used as a compliance test suite to test other implementations.

Test Vectors:
-------------

The test vectors are given as a list of tuples, each containing a URI template
and the URI that should result from processing the template. 
Processing a URI template requires a set of nodes from which template variable
values will be obtained. The test vectors contain a list of test nodes, each 
with node attributes and a uid. 

Reference Implementation:
-------------------------

This reference implementation implements the processing or URI templates given
a set of node objects. When a template variable refers to a choice between more
than one possible node, this implementation arbitrarily chooses the one that 
appears last in the list. Any other implementation under test should use the 
same choice if it wants to produce the same output, but this is not required 
by the specification.


Running the tests:
------------------

To run the tests, simply run this as a python script.
On a typical system, this can be done with a command line such as:

python marlin-URITemplates-CTS.py

The script will print the result of the test on the standard output.

'''

import re
import urlparse
import urllib

### Test vectors
TestNodes = [
    { 
        'uid':'urn:marlin:test:node-0001', 
        'attributes':{
            'urn:marlin:node:attribute:type':'user',
            'urn:marlin:organization:test:custom-attr':'SomeValue'
	    }
    },
    { 
        'uid':'urn:marlin:test:node-0002', 
        'attributes':{
            'urn:marlin:node:attribute:type':'user',
            'urn:marlin:organization:test:custom-attr':'SomeValue'
	    }
    },
    { 
        'uid':'urn:marlin:test:node-0003', 
        'attributes':{
            'urn:marlin:node:attribute:type':'user',
            'urn:marlin:organization:test:custom-attr':'SomeValue'
	    }
    },
    { 
        'uid':'urn:marlin:test:node-0004', 
        'attributes':{
            'urn:marlin:node:attribute:type':'personality',
            'urn:marlin:organization:test:custom-attr':'Some Value=='
	    }
    },
    { 
        'uid':'urn:marlin:test:node-0005', 
        'attributes':{
            'urn:marlin:node:attribute:type':'personality',
            'urn:marlin:organization:test:custom-attr':'SomeValue'
	    }
    },
    { 
        'uid':'urn:marlin:test:node-0006', 
        'attributes':{
            'urn:marlin:node:attribute:type':'urn:marlin:organization:test:MyType',
            'urn:marlin:organization:test:MyAttrName':'SomeValue'
	    }
    },
    { 
        'uid':'urn:marlin:test:node-with?strange#name', 
        'attributes':{
            'urn:marlin:node:attribute:type':'user',
            'urn:marlin:organization:test:MyAttrName':'Strange'
	    }
    }
]

TestTemplates = [
	('http://www.bok.net/test?user=foo','http://www.bok.net/test?user=foo'),
	('http://www.bok.net/test?user=%7Bfoo%7D','http://www.bok.net/test?user=%7Bfoo%7D'),
	('http://www.bok.net/test?user={m:urn:marlin:organization:test:MyAttrName=aGVsbG8=}','http://www.bok.net/test?user='),
	('http://www.bok.net/test?user={~m:urn:marlin:organization:test:MyAttrName=aGVsbG8=}','http://www.bok.net/test?user=~'),
	('http://www.bok.net/test?user={m:urn:marlin:organization:test:MyAttrName=SomeValue}','http://www.bok.net/test?user=urn%3Amarlin%3Atest%3Anode-0006'),
	('http://www.bok.net/test?user={m:type=personality&urn:marlin:organization:test:custom-attr=Some%20Value==}','http://www.bok.net/test?user=urn%3Amarlin%3Atest%3Anode-0004'),
	('http://www.bok.net/test?user={~m:type=personality&type=user}','http://www.bok.net/test?user=~'),
	('http://www.bok.net/test?user={z:foo}','http://www.bok.net/test?user='),
	('http://www.bok.net/test?user={~z:foo}','http://www.bok.net/test?user=~'),
	('http://www.bok.net/test?user={m:urn:marlin:organization:test:MyAttrName=Strange}','http://www.bok.net/test?user=urn%3Amarlin%3Atest%3Anode-with%3Fstrange%23name')
]

### regular expressions
VAR_PATTERN = re.compile(r'{(?P<var_modifier>~?)(?P<var_namespace>[a-zA-Z]):(?P<var_local_name>[a-zA-Z0-9\%\:\@\-\.\_\~\!\$\&\*\+\=]+)}')

### test if a node matches all the attribute/value pairs
def NodeMatches(node, pairs):
	for pair in pairs:
		attribute_name = pair[0] == 'type' and 'urn:marlin:node:attribute:type' or urllib.unquote(pair[0])
		attribute_value = urllib.unquote(pair[1])
		if not node['attributes'].has_key(attribute_name) or node['attributes'][attribute_name] != attribute_value:
			return False
	return True
	
### find a match for a list of attribute-name/attribute-value pairs
def GetMatch(pairs):
	nodes = [node for node in TestNodes if NodeMatches(node, pairs)]
	if len(nodes) == 0:
		# no match
		return None
	else:
		# one or more match. if there is more than one, we arbitrarily choose the last one
		return nodes[-1]['uid']
					
### replace a variable with its value
def ReplaceVar(var_name):
	(var_modifier, var_namespace, var_local_name) = var_name.group('var_modifier', 'var_namespace', 'var_local_name')
	if var_namespace == 'm':
		# marlin var namespace
		match = GetMatch([x.split('=',1) for x in var_local_name.split('&')])
		if match: return urllib.quote(match)
	
	# unknown namespace or no match
	return var_modifier == '~' and '~' or ''
	
### run one test case
def DoTest(entry):
	(template, expected) = entry
	processed = re.sub(VAR_PATTERN, ReplaceVar, template)
	if processed != expected:
		print 'WARNING: template', template, 'processsed ->', processed,' expected ->', expected
		return False
	return True
	
### main test
results = [DoTest(x) for x in TestTemplates]
print 'RESULTS: number of tests =', len(results), 'passed =',len([x for x in results if x])
